
#include "DSP2833x_EPwm_defines.h"
#include "DSP2833x_Device.h"
#include "DSP2833x_Examples.h"
#include "DSP2833x_eCAP_defines.h"
#include "ModbusSlave.h"

//################################################################################################################################################################################################
//						Prototype statements for functions found within this file.
//################################################################################################################################################################################################

void Gpio_select(void);

#ifndef DEBUG
// Do not remove code below used to put program on flash of DSP
// (must include 28335.cmd and exclude 28335_RAM_lnk.cmd)
void InitFlash(void);
#endif

void ADCconfig(void);
void Setup_ePWM(void);
void Setup_eCAP(void);
unsigned int spi_xmit(Uint16 a);
void spi_init(void);
void spi_fifo_init(void);
interrupt void cpu_timer0_isr(void);

extern void InitSysCtrl(void);
extern void InitPieCtrl(void);
extern void InitPieVectTable(void);
extern void InitCpuTimers(void);
extern void ConfigCpuTimer(struct CPUTIMER_VARS *, float, float);
extern void InitAdc(void);


//################################################################################################################################################################################################
//						Global Variables
//################################################################################################################################################################################################
ModbusSlave mb;

#ifndef DEBUG
// Do not remove code below used to put program on flash of DSP
// (must include 28335.cmd and exclude 28335_RAM_lnk.cmd)

extern unsigned int RamfuncsLoadStart;
extern unsigned int RamfuncsLoadEnd;
extern unsigned int RamfuncsRunStart;
#endif

//################################################################################################################################################################################################
//						main code									
//################################################################################################################################################################################################
void main(void)
{
	InitSysCtrl();	 																					// Basic Core Initialization
	EALLOW;
	SysCtrlRegs.WDCR = 0x0068;				 															// 0x00AF = Enable Watchdog 0x0068 = Disable Watchdog
	EDIS;

	DINT;																								// Disable all interrupts

#ifndef DEBUG
	// Do not remove code below used to put program on flash of DSP
	// (must include 28335.cmd and exclude 28335_RAM_lnk.cmd)
	memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, &RamfuncsLoadEnd - &RamfuncsLoadStart);				// Setup program to run in flash
	InitFlash();
#endif


	Gpio_select();																						// Setup GPIO's
	Setup_ePWM();																						// Setup ePWM signals
	InitPieCtrl();																						// Basic setup of PIE table; from DSP2833x_PieCtrl.c
	InitPieVectTable();																					// Default ISR's in PIE
	InitAdc();																							// Initialize Analog to Digital converter
	ADCconfig();																						// Configure the ADC
	spi_init();																							// Initialize SPI interface
	spi_fifo_init();																					// Initialize SPI FIFO
	Setup_eCAP();																						// Setup the eCAP pins

	EALLOW;																								// Enable access to protected register
	PieVectTable.TINT0 = &cpu_timer0_isr;																// Re-map entry for Timer0 from ESTOP0 to real interrupt service
	EDIS;																								// Disable access to protected register
	InitCpuTimers();																					// Basic setup CPU Timer0, 1 and 2
	ConfigCpuTimer(&CpuTimer0, 150, 100);																// Initialize Timer0 to 100us (150 = speed of DSP, 100000 = 100ms, 100 = 100us)
	PieCtrlRegs.PIEIER1.bit.INTx7 = 1;																	// Enable interrupt masks, Timer0 connected to group INT1, Bit7

	mb = construct_ModbusSlave();

	IER |= 1;																							// Enable interrupt core line 1 (INT 1), modify register IER acc
	EINT;																								// Enable control interrupts
	ERTM;																								// Enable debug interrupts

	CpuTimer0Regs.TCR.bit.TSS = 0;																		// Start Timer0 Interrupt


	while(1)																							// Main program (keep looping)
	{    
		mb.loopStates(&mb);

		mb.coils.dummy0=0;
		mb.inputRegisters.dummy0=0;
		mb.inputs.dummy0=0;
		mb.holdingRegisters.dummy0=0;
	}

} 

//################################################################################################################################################################################################
//						End of main code
//################################################################################################################################################################################################


//################################################################################################################################################################################################
//						Timer 0 (100us)
//################################################################################################################################################################################################

void cpu_timer0_isr()
{

	CpuTimer0.InterruptCount++;																			// Increment global interrupt counter
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}
//################################################################################################################################################################################################
//						End Timer 0 (100us)
//################################################################################################################################################################################################


//################################################################################################################################################################################################
//						GPIO Setup
//################################################################################################################################################################################################

void Gpio_select(void)
{
	EALLOW;
	GpioCtrlRegs.GPAMUX1.all = 0;																		// GPIO0  ... GPIO15 = General Purpose I/O


	GpioCtrlRegs.GPAMUX2.all = 0;																		// GPIO16 ... GPIO31 = General Purpose I/O
	GpioCtrlRegs.GPCMUX1.all = 0;																		// GPIO64 ... GPIO79 = General Purpose I/O
	GpioCtrlRegs.GPCMUX2.all = 0;																		// GPIO80 ... GPIO87 = General Purpose I/O

	GpioCtrlRegs.GPADIR.all = 0;																		// GPIO0  ... GPIO31 as inputs

	GpioCtrlRegs.GPBDIR.all = 0;																		// GPIO32 ... GPIO63 as inputs

	GpioCtrlRegs.GPCDIR.all = 0;																		// GPIO64 ... GPIO87 as inputs

	EDIS;
}
//################################################################################################################################################################################################
//						End of GPIO Setup
//################################################################################################################################################################################################


//################################################################################################################################################################################################
//						Setup ePWM
//################################################################################################################################################################################################

void Setup_ePWM(void)
{

	EPwm1Regs.TBCTL.bit.CLKDIV = TB_DIV1;																// CLKDIV = /1    ->default
	EPwm2Regs.TBCTL.bit.CLKDIV = TB_DIV1;																// CLKDIV = /1    ->default
	EPwm3Regs.TBCTL.bit.CLKDIV = TB_DIV1;																// CLKDIV = /1    ->default
	EPwm4Regs.TBCTL.bit.CLKDIV = TB_DIV1;																// CLKDIV = /1    ->default
	EPwm5Regs.TBCTL.bit.CLKDIV = TB_DIV1;																// CLKDIV = /1    ->default
	EPwm6Regs.TBCTL.bit.CLKDIV = TB_DIV1;																// CLKDIV = /1    ->default

	EPwm1Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;															// HSPCLKDIV = /1
	EPwm2Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;
	EPwm3Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;
	EPwm4Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;
	EPwm5Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;
	EPwm6Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;

	EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;															// Count up mode
	EPwm2Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;
	EPwm3Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;
	EPwm4Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;
	EPwm5Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;
	EPwm6Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;

	EPwm1Regs.TBCTL.bit.FREE_SOFT = 2;																	// Free Run (continue running when breakpoint is hit)
	EPwm2Regs.TBCTL.bit.FREE_SOFT = 2;
	EPwm3Regs.TBCTL.bit.FREE_SOFT = 2;
	EPwm4Regs.TBCTL.bit.FREE_SOFT = 2;
	EPwm5Regs.TBCTL.bit.FREE_SOFT = 2;
	EPwm6Regs.TBCTL.bit.FREE_SOFT = 2;

	// Set Up AQCTLA registers
	// When CTR = CMPA on Up Count - Clear
	// When CTR = CMPA on Up Count - Set
	// When CTR = 0 - Set
	// Modifying these can alter the direction of the duty cycle
	EPwm1Regs.AQCTLA.bit.CAU = AQ_CLEAR;
	EPwm1Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm1Regs.AQCTLA.bit.ZRO = AQ_SET;

	EPwm1Regs.AQCTLB.bit.CBU = AQ_CLEAR;
	EPwm1Regs.AQCTLB.bit.CBD = AQ_SET;
	EPwm1Regs.AQCTLB.bit.ZRO = AQ_SET;

	EPwm2Regs.AQCTLA.bit.CAU = AQ_CLEAR;
	EPwm2Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm2Regs.AQCTLA.bit.ZRO = AQ_SET;

	EPwm2Regs.AQCTLB.bit.CBU = AQ_CLEAR;
	EPwm2Regs.AQCTLB.bit.CBD = AQ_SET;
	EPwm2Regs.AQCTLB.bit.ZRO = AQ_SET;

	EPwm3Regs.AQCTLA.bit.CAU = AQ_CLEAR;
	EPwm3Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm3Regs.AQCTLA.bit.ZRO = AQ_SET;

	EPwm3Regs.AQCTLB.bit.CBU = AQ_CLEAR;
	EPwm3Regs.AQCTLB.bit.CBD = AQ_SET;
	EPwm3Regs.AQCTLB.bit.ZRO = AQ_SET;

	EPwm4Regs.AQCTLA.bit.CAU = AQ_CLEAR;
	EPwm4Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm4Regs.AQCTLA.bit.ZRO = AQ_SET;

	EPwm5Regs.AQCTLA.bit.CAU = AQ_CLEAR;
	EPwm5Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm5Regs.AQCTLA.bit.ZRO = AQ_SET;

	EPwm5Regs.AQCTLB.bit.CBU = AQ_CLEAR;
	EPwm5Regs.AQCTLB.bit.CBD = AQ_SET;
	EPwm5Regs.AQCTLB.bit.ZRO = AQ_SET;

	EPwm6Regs.AQCTLA.bit.CAU = AQ_CLEAR;
	EPwm6Regs.AQCTLA.bit.CAD = AQ_SET;
	EPwm6Regs.AQCTLA.bit.ZRO = AQ_SET;

	EPwm1Regs.CMPB 	= 3000;																				// Initialize green led here so there is no flash
	EPwm1Regs.TBPRD = 3000;																				// 50 kHz - PWM Signal  = TBPRD = fsysclockout/(fpwm*CLKDIV*HSPCLKDIV)

	EPwm2Regs.TBPRD = 3000;																				// TBPRD = 150e6/(50e3*1*1), CLKDIV and HSPCLKDIV are the divisor numbers both 1.

	EPwm3Regs.CMPB	= 3000;																				// Initialize red led here so there is no flash
	EPwm3Regs.TBPRD = 3000;																				// CTRMODE has a scaling factor of 0.5 if up/down mode is used

	EPwm4Regs.TBPRD = 3000;

	EPwm5Regs.CMPB	= 3000;																				// Initialize blue led here so there is no flash
	EPwm5Regs.TBPRD = 3000;

	EPwm6Regs.TBPRD = 3000;
}

//################################################################################################################################################################################################
//						End Setup ePWM1A
//################################################################################################################################################################################################


//################################################################################################################################################################################################
//						Setup eCAP
//################################################################################################################################################################################################
void Setup_eCAP(void)
{
	ECap1Regs.ECCTL1.bit.CAP1POL = EC_RISING;															// Capture Event 1 triggered on a rising edge (RE)
	ECap1Regs.ECCTL1.bit.CAP2POL = EC_RISING;															// Capture Event 2 triggered on a rising edge (RE)
	ECap1Regs.ECCTL1.bit.CAP3POL = EC_RISING;															// Capture Event 3 triggered on a rising edge (RE)
	ECap1Regs.ECCTL1.bit.CAP4POL = EC_RISING;															// Capture Event 4 triggered on a rising edge (RE)
	ECap1Regs.ECCTL1.bit.CTRRST1 = EC_ABS_MODE;															// Do not reset counter on Capture Event 1 (absolute time stamp)
	ECap1Regs.ECCTL1.bit.CTRRST2 = EC_ABS_MODE;															// Do not reset counter on Capture Event 2 (absolute time stamp)
	ECap1Regs.ECCTL1.bit.CTRRST3 = EC_ABS_MODE;															// Do not reset counter on Capture Event 3 (absolute time stamp)
	ECap1Regs.ECCTL1.bit.CTRRST4 = EC_ABS_MODE;															// Do not reset counter on Capture Event 4 (absolute time stamp)
	ECap1Regs.ECCTL1.bit.CAPLDEN = EC_ENABLE;															// Enable Loading of CAP1-4 registers on a capture event
	ECap1Regs.ECCTL1.bit.PRESCALE = EC_DIV1;
	ECap1Regs.ECCTL1.bit.FREE_SOFT = 2;																	// Free Run (continue running when breakpoint is hit)

	ECap1Regs.ECCTL2.bit.CONT_ONESHT = EC_ONESHT;														// Operate in one shot mode
	ECap1Regs.ECCTL2.bit.SYNCO_SEL = EC_SYNCO_DIS;														// Disable sync out signal
	ECap1Regs.ECCTL2.bit.SYNCI_EN = EC_DISABLE;															// Disable sync in signal
	ECap1Regs.ECCTL2.bit.TSCTRSTOP = EC_RUN;															// Counter stop control (free running)

	ECap2Regs.ECCTL1.bit.CAP1POL = EC_RISING;															// Capture Event 1 triggered on a rising edge (RE)
	ECap2Regs.ECCTL1.bit.CAP2POL = EC_RISING;															// Capture Event 2 triggered on a rising edge (RE)
	ECap2Regs.ECCTL1.bit.CAP3POL = EC_RISING;															// Capture Event 3 triggered on a rising edge (RE)
	ECap2Regs.ECCTL1.bit.CAP4POL = EC_RISING;															// Capture Event 4 triggered on a rising edge (RE)
	ECap2Regs.ECCTL1.bit.CTRRST1 = EC_ABS_MODE;															// Do not reset counter on Capture Event 1 (absolute time stamp)
	ECap2Regs.ECCTL1.bit.CTRRST2 = EC_ABS_MODE;															// Do not reset counter on Capture Event 2 (absolute time stamp)
	ECap2Regs.ECCTL1.bit.CTRRST3 = EC_ABS_MODE;															// Do not reset counter on Capture Event 3 (absolute time stamp)
	ECap2Regs.ECCTL1.bit.CTRRST4 = EC_ABS_MODE;															// Do not reset counter on Capture Event 4 (absolute time stamp)
	ECap2Regs.ECCTL1.bit.CAPLDEN = EC_ENABLE;															// Enable Loading of CAP1-4 registers on a capture event
	ECap2Regs.ECCTL1.bit.PRESCALE = EC_DIV1;
	ECap2Regs.ECCTL1.bit.FREE_SOFT = 2;																	// Free Run (continue running when breakpoint is hit)

	ECap2Regs.ECCTL2.bit.CONT_ONESHT = EC_ONESHT;														// Operate in one shot mode
	ECap2Regs.ECCTL2.bit.SYNCO_SEL = EC_SYNCO_DIS;														// Disable sync out signal
	ECap2Regs.ECCTL2.bit.SYNCI_EN = EC_DISABLE;															// Disable sync in signal
	ECap2Regs.ECCTL2.bit.TSCTRSTOP = EC_RUN;															// Counter stop control (free running)

	ECap5Regs.ECCTL1.bit.CAP1POL = EC_RISING;															// Capture Event 1 triggered on a rising edge (RE)
	ECap5Regs.ECCTL1.bit.CAP2POL = EC_RISING;															// Capture Event 2 triggered on a rising edge (RE)
	ECap5Regs.ECCTL1.bit.CAP3POL = EC_RISING;															// Capture Event 3 triggered on a rising edge (RE)
	ECap5Regs.ECCTL1.bit.CAP4POL = EC_RISING;															// Capture Event 4 triggered on a rising edge (RE)
	ECap5Regs.ECCTL1.bit.CTRRST1 = EC_ABS_MODE;															// Do not reset counter on Capture Event 1 (absolute time stamp)
	ECap5Regs.ECCTL1.bit.CTRRST2 = EC_ABS_MODE;															// Do not reset counter on Capture Event 2 (absolute time stamp)
	ECap5Regs.ECCTL1.bit.CTRRST3 = EC_ABS_MODE;															// Do not reset counter on Capture Event 3 (absolute time stamp)
	ECap5Regs.ECCTL1.bit.CTRRST4 = EC_ABS_MODE;															// Do not reset counter on Capture Event 4 (absolute time stamp)
	ECap5Regs.ECCTL1.bit.CAPLDEN = EC_ENABLE;															// Enable Loading of CAP1-4 registers on a capture event
	ECap5Regs.ECCTL1.bit.PRESCALE = EC_DIV1;
	ECap5Regs.ECCTL1.bit.FREE_SOFT = 2;																	// Free Run (continue running when breakpoint is hit)

	ECap5Regs.ECCTL2.bit.CONT_ONESHT = EC_ONESHT;														// Operate in one shot mode
	ECap5Regs.ECCTL2.bit.SYNCO_SEL = EC_SYNCO_DIS;														// Disable sync out signal
	ECap5Regs.ECCTL2.bit.SYNCI_EN = EC_DISABLE;															// Disable sync in signal
	ECap5Regs.ECCTL2.bit.TSCTRSTOP = EC_RUN;															// Counter stop control (free running)
}

//################################################################################################################################################################################################
//						End of Setup eCAP
//################################################################################################################################################################################################


//################################################################################################################################################################################################
//						ADC Config
//################################################################################################################################################################################################
void ADCconfig()
{
	AdcRegs.ADCTRL1.all = 0;
	AdcRegs.ADCTRL1.bit.ACQ_PS = 7;																		// Sampling sw close time = (2 / HSPCLK) * 8 - ends up = .1us
	AdcRegs.ADCTRL1.bit.SEQ_CASC = 0; 																	// 0 = Dual Sequencer Mode, 1 = cascaded sequencer
	AdcRegs.ADCTRL1.bit.CPS = 0;																		// divide by 1
	AdcRegs.ADCTRL1.bit.CONT_RUN = 0; 																	// Single Run Mode
	AdcRegs.ADCTRL2.all = 0;
	AdcRegs.ADCTRL2.bit.INT_ENA_SEQ1 = 0;																// Disable SEQ1 interrupt
	AdcRegs.ADCTRL3.bit.ADCCLKPS = 6;																	// ADC clock: FCLK = HSPCLK / (2 * ADCCLKPS)
																										// HSPCLK = 150MHz (see DSP2833x_SysCtrl.c)
																										// FCLK = 12.5 MHz
	AdcRegs.ADCTRL3.bit.SMODE_SEL = 1;																	// Simultaneous Sampling Mode
	AdcRegs.ADCMAXCONV.all = 0x0003;																	// 4 conversions from Sequencer 1 (Number of conversions minus 1)
																										// Really 8 b/c ADC is in Simultaneous Sampling mode
																										// When in Dual Sequencer Mode ADCMAXCONV.all is bits 0-2 are for SEQ1
																										// bits 4-6 are for SEQ2
	AdcRegs.ADCCHSELSEQ1.bit.CONV00 = 0;																// RESULT0 = ADCINA0 = VOLTS, 			RESULT1 = ADCINB0 = CURRENT
	AdcRegs.ADCCHSELSEQ1.bit.CONV01 = 1;																// RESULT2 = ADCINA1 = I_MOT, 			RESULT3 = ADCINB1 = PHASE
	AdcRegs.ADCCHSELSEQ1.bit.CONV02 = 7;																// RESULT4 = ADCINA7 = PHASE_ERROR, 	RESULT5 = ADCINB7 = VCO_IN
	AdcRegs.ADCCHSELSEQ1.bit.CONV03 = 6;																// RESULT6 = ADCINA6 = AGC_ERROR, 		RESULT7 = ADCINB6 = AGC_PWM
																										// This is because ADC is in Simultaneous Sampling Mode
}
//################################################################################################################################################################################################
//						End ADC Config
//################################################################################################################################################################################################


//################################################################################################################################################################################################
//						SPI Init
//################################################################################################################################################################################################
void spi_init()
{
	SpiaRegs.SPICCR.all =0x000F;	             														// Reset on, rising edge, 16-bit char bits
	SpiaRegs.SPICTL.all =0x0006;    		    														// Enable master mode, normal phase,
                                                 	 	 	 	 	 	 	 	 	 	 	 	 	 	// Enable talk, and SPI int disabled.
	SpiaRegs.SPIBRR =0x007F;
    SpiaRegs.SPICCR.all =0x008F;		         														// Relinquish SPI from Reset
    SpiaRegs.SPIPRI.bit.FREE = 1;                														// Set so breakpoints don't disturb xmission
}
//################################################################################################################################################################################################
//						End SPI Init
//################################################################################################################################################################################################


//################################################################################################################################################################################################
//						SPI FIFO Init
//################################################################################################################################################################################################
void spi_fifo_init()
{
    SpiaRegs.SPIFFTX.all=0xE040;
    SpiaRegs.SPIFFRX.all=0x2044;
    SpiaRegs.SPIFFCT.all=0x0;
}
//################################################################################################################################################################################################
//						End SPI FIFO Init
//################################################################################################################################################################################################
